# coding:utf-8

import sys
import os
import shutil
import getopt
import requests
import json
from termcolor import colored
from spider import YoudaoSpider
from model import Word
import config


def query(keyword, use_db=True, use_api=False, play_voice=False):
    word = Word.get_word(keyword)
    if use_db and word:
        YoudaoSpider.show_result(json.loads(word.json_data))
    else:
        if not word:
            word = Word()
        spider = YoudaoSpider(keyword)
        try:
            result = spider.get_result(use_api)
            spider.show_result()
            word.keyword = keyword
            word.json_data = json.dumps(result)
            word.save()
        except requests.HTTPError, e:
            print colored(u'网络错误: %s' % e.message, 'red')
            sys.exit()

    if play_voice:
        YoudaoSpider.get_voice(keyword)


def show_db_list():
    print colored(u'保存在数据库中的单词及查询次数:', 'blue')
    for word in Word.select():
        print(colored(word.keyword, 'cyan'), colored(str(word.count), 'green'))


def del_word(keyword):
    if keyword:
        try:
            word = Word.select().where(Word.keyword == keyword).get()
            word.delete_instance()
            print(colored(u'已删除{0}'.format(keyword), 'blue'))
        except Word.DoesNotExist:
            print(colored(u'没有找到{0}'.format(keyword), 'red'))
        config.silent_remove(os.path.join(config.VOICE_DIR, keyword+'.mp3'))
    else:
        count = Word.delete().execute()
        shutil.rmtree(config.VOICE_DIR, ignore_errors=True)
        print(colored(u'共删除{0}个单词'.format(count), 'blue'))


def show_help():
    print(u"""
    控制台下的有道词典 版本{ver}
    默认通过解析有道网页版获取查询结果, 没有词典结果时自动使用有道翻译,
    查询结果会保存到sqlite 数据库中
    使用方法 yd word [-a] [-n] [-l] [-c] [-v] [-d word] [--help]
    [-a] 使用API 而不是解析网页获取结果
    [-n] 强制重新获取, 不管数据库中是否已经保存
    [-l] 列出数据库中保存的所有单词
    [-c] 清空数据库
    [-v] 获取单词发音, 单独使用 yd -v 可以获取上一个查询单词的发音
    [-d word] 删除数据库中某个单词
    [--help] 显示帮助信息
    """.format(ver=config.VERSION))


def main():
    config.prepare()
    try:
        options, args = getopt.getopt(sys.argv[1:], 'anld:cv', ['help'])
    except getopt.GetoptError:
        options = [('--help', '')]
    if ('--help', '') in options:
        show_help()
        return

    use_api = False
    use_db = True
    play_voice = False
    for opt in options:
        if opt[0] == '-a':
            use_api = True
        elif opt[0] == '-n':
            use_db = False
        elif opt[0] == '-l':
            show_db_list()
            return
        elif opt[0] == '-d':
            del_word(opt[1])
            return
        elif opt[0] == '-c':
            del_word(None)
            return
        elif opt[0] == '-v':
            play_voice = True

    keyword = ' '.join(args)

    # 播放上一个单词的声音
    if play_voice and not keyword:
        word = Word.get_last_word()
        keyword = word.keyword
        query(keyword, play_voice=True, use_db=True)
    else:
        while not keyword:
            keyword = raw_input(colored('input a word: ', 'blue'))

        query(keyword, use_db, use_api, play_voice)

if __name__ == '__main__':
    main()


# coding: utf-8

import json
import datetime
from peewee import *
from config import DB_DIR


db = SqliteDatabase(DB_DIR)


class BaseModel(Model):
    class Meta:
        database = db


class Word(BaseModel):
    keyword = CharField(index=True, unique=True)
    json_data = TextField()
    add_time = DateTimeField(default=datetime.datetime.now())
    query_time = DateTimeField(default=datetime.datetime.now())
    count = IntegerField(default=1)

    @classmethod
    def get_word(cls, keyword):
        try:
            word = cls.select().where(Word.keyword == keyword).get()
            word.query_time = datetime.datetime.now()
            word.count += 1
            word.save()
            return word
        except Word.DoesNotExist:
            return None

    @classmethod
    def get_last_word(cls):
        try:
            word = cls.select().order_by(cls.query_time.desc()).get()
            return word
        except cls.DoesNotExist:
            return None


db.create_tables([Word], safe=True)



# coding: utf-8

import os
import errno
import pickle

VERSION = '0.2.0'
HOME = os.path.expanduser("~")
BASE_DIR = os.path.join(HOME, '.dict_youdao')   # 用户数据根目录
VOICE_DIR = os.path.join(BASE_DIR, 'voice')     # 音频文件

DATABASE = 'youdao.db'
PK_FILE = 'youdao.pk'
DB_DIR = os.path.join(BASE_DIR, DATABASE)
PK_DIR = os.path.join(BASE_DIR, PK_FILE)


def update():
    # 从0.2.0开始更改了数据库
    # 重新设置数据库
    silent_remove(DB_DIR)


def silent_remove(filename):
    try:
        os.remove(filename)
    except OSError, e:
        if e.errno != errno.ENOENT:
            raise


def prepare():
    if not os.path.exists(BASE_DIR):
        os.mkdir(BASE_DIR)
    if not os.path.exists(VOICE_DIR):
        os.mkdir(VOICE_DIR)

    config = {'version': '0'}
    if os.path.isfile(PK_DIR):
        with open(PK_DIR, 'rb') as f:
            config = pickle.load(f)
    # update
    if config.get('version', '0') < '0.2.0':
        update()
        with open(PK_DIR, 'wb') as f:
            config['version'] = VERSION
            pickle.dump(config, f)


# coding:utf-8

import re
import json
import os
import requests
import webbrowser
from termcolor import colored
from bs4 import BeautifulSoup
from config import VOICE_DIR


class YoudaoSpider:
    """
    通过有道获取单词解释, 以及展示查询结果
    """

    params = {
        'keyfrom': 'longcwang',
        'key': '131895274',
        'type': 'data',
        'doctype': 'json',
        'version': '1.1',
        'q': 'query'
    }
    api_url = 'http://fanyi.youdao.com/openapi.do'
    voice_url = 'http://dict.youdao.com/dictvoice?type=2&audio={word}'
    web_url = 'http://dict.youdao.com/search?keyfrom=dict.top&q='
    translation_url = 'http://fanyi.youdao.com/translate?keyfrom=dict.top&i='

    error_code = {
        0: u'正常',
        20: u'要翻译的文本过长',
        30: u'无法进行有效的翻译',
        40: u'不支持的语言类型',
        50: u'无效的key',
        60: u'无词典结果，仅在获取词典结果生效'
    }

    result = {
        "query": "",
        "errorCode": 0,
    }

    def __init__(self, word):
        self.word = word

    @classmethod
    def show_result(cls, result=None):
        """
        展示查询结果
        :param result: 与有道API返回的json 数据结构一致的dict
        """
        if not result:
            result = cls.result
        if result['errorCode'] != 0:
            print colored(cls.error_code[result['errorCode']], 'red')
        else:
            print colored('[%s]' % result['query'], 'magenta')
            if 'basic' in result:
                if 'us-phonetic' in result['basic']:
                    print colored(u'美音:', 'blue'), colored('[%s]' % result['basic']['us-phonetic'], 'green'),
                if 'uk-phonetic' in result['basic']:
                    print colored(u'英音:', 'blue'), colored('[%s]' % result['basic']['uk-phonetic'], 'green')
                if 'phonetic' in result['basic']:
                    print colored(u'拼音:', 'blue'), colored('[%s]' % result['basic']['phonetic'], 'green')

                print colored(u'基本词典:', 'blue')
                print colored('\t'+'\n\t'.join(result['basic']['explains']), 'yellow')

            if 'translation' in result:
                print colored(u'有道翻译:', 'blue')
                print colored('\t'+'\n\t'.join(result['translation']), 'cyan')

            if 'web' in result:
                print colored(u'网络释义:', 'blue')
                for item in result['web']:
                    print '\t' + colored(item['key'], 'cyan') + ': ' + '; '.join(item['value'])

    def get_result(self, use_api=False):
        """
        获取查询结果
        :param use_api:是否使用有道API, 否则解析web版有道获取结果
        :return:与有道API返回的json数据一致的dict
        """
        if use_api:
            self.params['q'] = self.word
            r = requests.get(self.api_url, params=self.params)
            r.raise_for_status()    # a 4XX client error or 5XX server error response
            self.result = r.json()
        else:
            r = requests.get(self.web_url + self.word)
            r.raise_for_status()
            self.parse(r.text)
        return self.result

    def parse(self, html):
        """
        解析web版有道的网页
        :param html:网页内容
        :return:result
        """
        soup = BeautifulSoup(html)
        root = soup.find(id='results-contents')

        # query 搜索的关键字
        keyword = root.find(class_='keyword')
        if not keyword:
            self.result['query'] = self.word
        else:
            self.result['query'] = unicode(keyword.string)

        # 基本解释
        basic = root.find(id='phrsListTab')
        if basic:
            trans = basic.find(class_='trans-container')
            if trans:
                self.result['basic'] = {}
                self.result['basic']['explains'] = [unicode(tran.string) for tran in trans.find_all('li')]
                # 中文
                if len(self.result['basic']['explains']) == 0:
                    exp = trans.find(class_='wordGroup').stripped_strings
                    self.result['basic']['explains'].append(' '.join(exp))

                # 音标
                phons = basic(class_='phonetic', limit=2)
                if len(phons) == 2:
                    self.result['basic']['uk-phonetic'], self.result['basic']['us-phonetic'] = \
                        [unicode(p.string)[1:-1] for p in phons]
                elif len(phons) == 1:
                    self.result['basic']['phonetic'] = unicode(phons[0].string)[1:-1]

        # 翻译
        if 'basic' not in self.result:
            self.result['translation'] = self.get_translation(self.word)

        # 网络释义(短语)
        web = root.find(id='webPhrase')
        if web:
            self.result['web'] = [
                {
                    'key': unicode(wordgroup.find(class_='search-js').string).strip(),
                    'value': [v.strip() for v in unicode(wordgroup.find('span').next_sibling).split(';')]
                } for wordgroup in web.find_all(class_='wordGroup', limit=4)
            ]

    def get_translation(self, word):
        """
        通过web版有道翻译抓取翻译结果
        :param word:str 关键字
        :return:list 翻译结果
        """
        r = requests.get(self.translation_url+word)
        if r.status_code != requests.codes.ok:
            return None

        pattern = re.compile(r'"translateResult":\[(\[.+\])\]')
        m = pattern.search(r.text)
        result = json.loads(m.group(1))
        return [item['tgt'] for item in result]

    @classmethod
    def get_voice(cls, word):
        print(colored(u'获取发音:{word}'.format(word=word), 'green'))

        voice_file = os.path.join(VOICE_DIR, word+'.mp3')
        if not os.path.isfile(voice_file):
            r = requests.get(cls.voice_url.format(word=word))
            with open(voice_file, 'wb') as f:
                f.write(r.content)
        print(colored(u'获取成功,播放中...', 'green'))

        saveout1 = os.dup(1)
        saveout2 = os.dup(2)
        os.close(1)
        os.close(2)
        try:
            webbrowser.open(voice_file)
        finally:
            os.dup2(saveout1, 1)
            os.dup2(saveout2, 2)


if __name__ == '__main__':
    test = YoudaoSpider('application')
    print test.get_result()